## calcnoise.cl
## Yosuke Minowa (2008/06/30)

procedure calcnoise (image, raperture, region)

file image     {prompt = "Image file name."}
real raperture {prompt = "Aperture radius"}
string region  {prompt = "Image region for noise estimation"}
int  maxaper   {500, prompt = "Maximum number of apertures"}
int  nhistbin  {30, prompt = "Number of bins in histogram"}
real  hist_z1   {INDEF, prompt = "Min bin value in histogram"}
real  hist_z2   {INDEF, prompt = "Max bin value in histogram"}
struct *list1  {mode = 'h'}
struct *list2  {mode = 'h'}

begin
	char _image, _region
	char tmp_image, tmp_xcoo, tmp_ycoo, tmp_coo, tmp_phot
	char tmp_flux, tmp_hist, tmp_hist2, tmp_fitres, tmp_fitparam
	char tmp1,tmp2,tmp3,tmp4,tmp5,tmp6
	char head, flux
	int amari
	real xcoo, ycoo
	real fwhm, sigma
	real rap, count, num
	int xwidth, ywidth
	real area, fwhm_ini, peak
	real h_z1, h_z2
	int len, idx
	int apnum, scale
	int maxap, buffer, nbin

	_image = image
	_region = region
	rap = raperture
	maxap = maxaper
	nbin = nhistbin
	h_z1 = hist_z1
	h_z2 = hist_z2

	tmp_image = mktemp("tmp_img.")
	imcopy(_image//_region, tmp_image, ver-)
	
	hselect(tmp_image, "NAXIS1", yes) | scan(xwidth)
	hselect(tmp_image, "NAXIS2", yes) | scan(ywidth)

	area = xwidth * ywidth * 0.8
	apnum = int(area / (rap*rap*3.1416))

	if(apnum > maxap){
	  apnum = maxap
	}

	buffer = int(rap) * 2
	scale = xwidth - 2 * buffer
	tmp_xcoo = mktemp("tmp_file.")
	urand (apnum, 1, ndigits=5, seed=1, scale_factor=scale, >> tmp_xcoo)
	scale = ywidth - 2 * buffer
	tmp_ycoo = mktemp("tmp_file.")
	urand (apnum, 1, ndigits=5, seed=2, scale_factor=scale, >> tmp_ycoo)

	tmp_coo = mktemp("tmp_file.")
	list1 = tmp_xcoo
	list2 = tmp_ycoo
	while(fscan(list1, xcoo) != EOF){
            if(fscan(list2, ycoo) != EOF){
	        xcoo = xcoo + buffer + 1
		ycoo = ycoo + buffer + 1
	        print(xcoo,ycoo,>> tmp_coo)
            }
	}
	del(tmp_xcoo, ver-)
	del(tmp_ycoo, ver-)

	tmp_phot = mktemp("tmp_file.")
	phot (tmp_image, "", coords=tmp_coo, output=tmp_phot,
	plotfile="", scale=1., fwhmpsf=INDEF, emission=yes,
	sigma=54.11, datamin=INDEF, datamax=INDEF, noise="poisson",
	ccdread="", gain="", readnoise=0., epadu=1., exposure="",
	airmass="", filter="", obstime="", itime=1., xairmass=INDEF,
	ifilter="INDEF", otime="INDEF", calgorithm="none", cbox=5.,
	cthreshold=0., minsnratio=1., cmaxiter=10, maxshift=1.,
	clean=no, rclean=1., rclip=2., kclean=3., mkcenter=no,
	salgorithm="median", annulus=rap, dannulus=5, skyvalue=0,
	smaxiter=10, sloclip=0., shiclip=0., snreject=50,
	sloreject=3., shireject=3., khist=3., binsize=0.1, smooth=no,
	rgrow=0., mksky=no, weighting="constant", apertures=rap,
	zmag=25, mkapert=no, interactive=no, radplots=no,
	icommands="", gcommands="", wcsin=")_.wcsin",
	wcsout=")_.wcsout", cache=")_.cache", verify=no, update=yes,
	verbose=")_.verbose",graphics=")_.graphics",
	display=")_.display")

	del(tmp_coo, ver-)
	imdel(tmp_image)

	tmp_flux = mktemp("tmpp_file.")
	list = tmp_phot
	i = 0
	while(fscan(list,line) != EOF){
          head = substr(line, 1, 1)
	  if(head != "#"){
	    i += 1
	    amari = i % 5
	    if(amari == 0){
	      flux = substr(line, 37, 47)
	      print(flux, >> tmp_flux)
	    }
          }
	}
	cat(tmp_flux, >> "test.dat") 
	del(tmp_phot, ver-)

	histogram (tmp_flux, colname="1", nbins=nbin, z1=h_z1, z2=h_z2, listout=no, title="Histogram: ", xlabel="Data values", ylabel="Count", logplot=no, fulline=yes, cumulative=no)

	tmp_hist = mktemp("tmpp_file.")
	histogram (tmp_flux, colname="1", nbins=nbin, z1=h_z1, z2=h_z2, listout=yes, logplot=no, fulline=yes, cumulative=no, >> tmp_hist)
	del(tmp_flux, ver-)
	
	list = tmp_hist
	tmp_hist2 = mktemp("tmp_file.")
	i = 0
	while(fscan(list,count,num) != EOF){
	  if(count < 0){
            if(i == 0){
	      fwhm_ini = count * (-0.5)
	    }
	    print(count,num,>>tmp_hist2)
	    i += 1
	    peak = num
	  }
        }
	del(tmp_hist, ver-)

	tmp_fitres = mktemp("tmp_file.")

	unlearn("cgausspars")
	unlearn("errorpars")
	unlearn("controlpars")
	unlearn("samplepars")
	cgausspars.ampl1 = peak
	cgausspars.cent1 = 0
	cgausspars.fwhm1 = fwhm_ini
	cgausspars.ampl2 = 0
	cgausspars.cent2 = 0
	cgausspars.fwhm2 = 0
	cgausspars.ampl3 = 0
	cgausspars.cent3 = 0
	cgausspars.fwhm3 = 0
	cgausspars.vampl1 = yes
	cgausspars.vcent1 = no
	cgausspars.vfwhm1 = yes
	cgausspars.vampl2 = no
	cgausspars.vcent2 = no
	cgausspars.vfwhm2 = no
	cgausspars.vampl3 = no
	cgausspars.vcent3 = no
	cgausspars.vfwhm3 = no

	ngaussfit (tmp_hist2, tmp_fitres, function="cgauss", rt=no,tablein="", row=1, interactive=no,verbose=no, pcomp=no,ltype="continuous", device="stdgraph", gcur="")
	
	tmp_fitparam = 	mktemp("tmp_file.")
	prfit(tmp_fitres, >> tmp_fitparam)

	list = tmp_fitparam
	while(fscan(list,line) != EOF){
	  head = substr(line, 1, 3)
	  if(head == "pos"){
	    len = strlen(line)
	    idx = stridx("=", line)
	    line = substr(line, idx+1, len)

	    len = strlen(line)
	    idx = stridx("=", line)
	    line = substr(line, idx+1, len)

	    len = strlen(line)
	    idx = stridx("=", line)
	    line = substr(line, idx+1, len)
	    print(line) | scan(fwhm)
	  }
	}

	sigma = fwhm / 2.3548

	print("\n### Result")
	printf("#1 sigma[adu] in aperture radius %.2f[pix]\n",rap)
	printf("%.2f\n", sigma)

	del(tmp_hist2, ver-)
	del(tmp_fitres, ver-)
	del(tmp_fitparam, ver-)

end
