#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<fitsio.h>

#define pi 3.14159265358979

int writeimg (char *fname, double *input, long imwidth, long imheight){
  fitsfile *fptr;
  int status = 0;
  int naxis=2;
  long fpixel[2] = {1, 1};
  long npixels;
  long naxes[2];
  long i,j;
  double *pix;

  naxes[0] = imwidth;
  naxes[1] = imheight;

  npixels = naxes[0] * naxes[1];

  fits_create_file(&fptr, fname, &status);
  if(status)
    return(status);
  fits_create_img(fptr, DOUBLE_IMG, naxis, naxes, &status);
  if(status)
    return(status);
  
  pix = (double *) malloc(naxes[0] * sizeof(double));

  fpixel[0] = 1;

  for(j=1; j<=naxes[1]; j++){
    fpixel[1] = j;
    for(i=0; i<naxes[0]; i++){
      pix[i] = input[(j-1)*naxes[0] + i];
    }
    fits_write_pix(fptr, TDOUBLE, fpixel, naxes[0], pix, &status);
    if(status)
      return(status);
  }
  
  fits_close_file(fptr, &status);
  free(pix);
  
  return(status);
}

int main(int argc, char *argv[]){
  long i,j;
  long imwidth, imheight, size;
  double dx,dy;
  double *phase;
  int status = 0;

  if(argc != 5){
    printf("Usage: %s [x-shift] [y-shift] [image size(pix)] [output]\n",argv[0]);
    return(1);
  }

  dx = atof(argv[1]);
  dy = atof(argv[2]);
  size = atoi(argv[3]);

  imwidth = size;
  imheight = size;

  phase = (double *) malloc(sizeof(double) * imwidth * imheight);

  for(j=0;j<imheight;j++){
    for(i=0;i<imwidth;i++){
      phase[j*imwidth+i] = (i*dx*2*pi)/imwidth + (j*dy*2*pi)/imheight;
    }
  }


  status = writeimg(argv[4], phase, imwidth, imheight);
  fits_report_error(stderr, status);
 
  free(phase);

  return(0);
}
  
