#include<stdio.h>
#include<math.h>
#include<fitsio.h>
#include<stdlib.h>
#include<string.h>

int writeimg (char *fname, double *input, long imwidth, long imheight){
  fitsfile *fptr;
  int status = 0;
  int naxis=2;
  long fpixel[2] = {1, 1};
  long npixels;
  long naxes[2];
  long i,j;
  double *pix;

  naxes[0] = imwidth;
  naxes[1] = imheight;

  npixels = naxes[0] * naxes[1];

  fits_create_file(&fptr, fname, &status);
  if(status)
    return(status);
  fits_create_img(fptr, DOUBLE_IMG, naxis, naxes, &status);
  if(status)
    return(status);
  
  pix = (double *) malloc(naxes[0] * sizeof(double));

  fpixel[0] = 1;

  for(j=1; j<=naxes[1]; j++){
    fpixel[1] = j;
    for(i=0; i<naxes[0]; i++){
      pix[i] = input[(j-1)*naxes[0] + i];
    }
    fits_write_pix(fptr, TDOUBLE, fpixel, naxes[0], pix, &status);
    if(status)
      return(status);
  }
  
  free(pix);

  fits_close_file(fptr, &status);
  return(status);
}

int main(int argc, char *argv[]){
  long imwidth, imheight;
  long size;
  long i,j;
  double lambda;
  double scale,pscale;
  double r,x,y,xc,yc;
  double A_max,A_min;
  double B_max,B_min;
  double C_max,C_min;
  double D_max,D_min;
  double X,Y;
  double D_p,D_s, offset, angle, sp_thick;
  double *pupil;
  double test;
  int status;

  if(argc != 5){
    printf("Usage: %s [wl(um)] [pixel scale(\"/pix)] [image size(pix)] [output]\n",argv[0]);
    return(1);
  }

  /*** telescope parameters ***/
  D_p = 7.92223;  /* diameter of primary mirror [m] */
  D_s = 2.59767; /* diameter of secondary mirror [m] */
  offset = 1.278; /* spider offset [m] */
  angle = 51.75; /* spider angle [deg]*/
  sp_thick = 0.224; /* Spider thickness [m] */

  /* deg --> rad */
  angle = angle * 0.01745329252;
  //  printf("%lf %lf\n",tan(angle),sin(angle));

  lambda = atof(argv[1]);
  pscale = atof(argv[2]);
  size = atoi(argv[3]);

  imwidth = size;
  imheight = size;

  /* image center */
  xc = (double)size * 0.5 + 0.5;
  yc = (double)size * 0.5 + 0.5;

  /* distance scale on pupil image */
  /* (size) * (scale) * (pscale) * (2pi/lambda) = 2pi */
  scale = 0.2062648062 * lambda / (size * pscale); 
  
  /* memory allocation for pupil image */
  pupil = (double *) malloc(imwidth * imheight * sizeof(double));

  for(j=0;j<imheight;j++){
    for(i=0;i<imwidth;i++){
      // printf("%d %d\n",i,j);

      x = (double)i;
      y = (double)j;

      X = (x-xc) * scale;
      Y = (y-yc) * scale;

      r = sqrt(X*X + Y*Y);

      if(r > D_s/2 && r <= D_p/2){
	if(Y>0){
	  A_max = (Y/tan(angle) + offset*0.5) + sp_thick / (2*sin(angle));
	  A_min = (Y/tan(angle) + offset*0.5) - sp_thick / (2*sin(angle));
	  D_max = -1*(Y/tan(angle) + offset*0.5) + sp_thick / (2*sin(angle));
	  D_min = -1*(Y/tan(angle) + offset*0.5) - sp_thick / (2*sin(angle));
	  if(X <= A_max && X >= A_min){
	    pupil[j*imwidth+i] = 0;
	  }else if(X <= D_max && X >= D_min){
	    pupil[j*imwidth+i] = 0;
	  }else{
	    pupil[j*imwidth + i] = 1;
	  }
	  
	}else if(Y<=0){
	  B_max = (-Y/tan(angle) + offset*0.5) + sp_thick / (2*sin(angle));
	  B_min = (-Y/tan(angle) + offset*0.5) - sp_thick / (2*sin(angle));
	  C_max = (Y/tan(angle) - offset*0.5) + sp_thick / (2*sin(angle));
	  C_min = (Y/tan(angle) - offset*0.5) - sp_thick / (2*sin(angle));
	  if(X <= B_max && X >= B_min){
	    pupil[j*imwidth+i] = 0;
	  }else if(X <= C_max && X >= C_min){
	    pupil[j*imwidth+i] = 0;
	  }else{
	    pupil[j*imwidth + i] = 1;
	  }
	}
      }else{
	pupil[j*imwidth + i] = 0;	
      }
    }
  } 
    
  status = writeimg(argv[4],pupil,imwidth,imheight);
  fits_report_error(stderr, status);

  free(pupil);

  return(0);
}

